use std::sync::Arc;

use hyper::server::conn::http1;
use hyper_util::rt::TokioIo;
use tokio::net::TcpListener;
use std::sync::Mutex;

mod filegetter;
mod sni;
mod env;

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error + Send + Sync>> {
    let env = env::Env::default();

    println!("Running on {}", env.listen);

    let listener = TcpListener::bind(env.listen).await?;
    let files = Arc::new(Mutex::new(filegetter::FileGetter::new()));

    loop {
        let (stream, _) = listener.accept().await?;
        let client_ip = stream.peer_addr().map(|x| Some(x)).unwrap_or(None);

        let io = TokioIo::new(stream);
        let env = env.clone();
        let files = files.clone();

        tokio::task::spawn(async move {
            let svc = sni::SniWebsitesService {
                client_ip,
                env: env.clone(),
                files
            };

            if let Err(e) = http1::Builder::new()
                .serve_connection(io, svc)
                .await {
                    eprintln!("Error while serving connection:");
                    eprintln!("{e:#?}");
            }
        });
    }
}
